/*
 * Copyright 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * DemCatalog implementation.
 */

#include <DemUnidraw/demcatalog.h>
#include <DemUnidraw/demcomps.h>
#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/maproute.h>

#include <OverlayUnidraw/ovfile.h>
#include <Attribute/paramlist.h>
#include <stdio.h>
#include <stream.h>
#include <ctype.h>

/*****************************************************************************/

DemCatalog::DemCatalog (
    const char* name, Creator* creator
) : MapCatalog(name, creator) {
}

boolean DemCatalog::Retrieve (const char* name, Component*& comp) {
    if (Valid(name, comp)) {
        _valid = true;

    } else if (UnidrawFormat(name)) {
        _valid = Catalog::Retrieve(name, comp);

    } else {
        filebuf fbuf;
        _valid = fbuf.open(name, input) != 0;

        if (_valid) {
	    istream in(&fbuf);
	    const int len = BUFSIZ;
	    char buf[len];

	    char ch;
	    while (isspace(ch = in.get())); in.putback(ch);
	    ParamList::parse_token(in, buf, len);
	    if (strcmp(buf, "mapfcl") == 0) { 
		comp = new MapFClassComp(in, _parent);
		_valid = in.good() && ((OverlayComp*)comp)->valid();
	    } else if (strcmp(buf, "mapserv") == 0) { 
		comp = new DemIdrawComp(in, name);
		_valid = in.good() && ((OverlayComp*)comp)->valid();
	    } else 
		_valid = false;

            if (_valid) {
                Forget(comp, name);
                Register(comp, name);
            }
        }
    }
    return _valid;
}

OverlayComp* DemCatalog::ReadComp(const char* name, istream& in, OverlayComp* parent) {
  OverlayComp* child = nil;

  if (strcmp(name, "surface") == 0 )
    child = new SurfaceComp(in, parent);
  else if (strcmp(name, "roads") == 0 )
    child = new RoadsComp(in, parent);
  else 
    child = MapCatalog::ReadComp(name, in, parent);
  return child;
}
