/*
 * Copyright (c) 1996 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifdef VTK

#ifndef MAKEDEPEND
#include <vtkRenderWindow.h>
#endif

#include <DemUnidraw/demcmds.h>
#include <DemUnidraw/demeditor.h>
#include <DemUnidraw/demkit.h>
#include <DemUnidraw/demviewer.h>
#include <MapUnidraw/mapabout.h>
#include <MapUnidraw/mapimport.h>
#include <OverlayUnidraw/ovcmds.h>
#include <OverlayUnidraw/ovpage.h>
#include <IVGlyph/bdfltform.h>
#include <IVGlyph/importchooser.h>
#include <UniIdraw/idkybd.h>
#include <Unidraw/catalog.h>
#include <Unidraw/ctrlinfo.h>
#include <Unidraw/grid.h>
#include <Unidraw/keymap.h>
#include <Unidraw/kybd.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/Graphic/pspaint.h>
#include <InterViews/background.h>
#include <InterViews/canvas.h>
#include <InterViews/frame.h>
#include <InterViews/layout.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/target.h>
#include <InterViews/window.h>
#include <IV-look/kit.h>
#include <IV-look/menu.h>
#include <OS/string.h>
#include <stdlib.h>
#include <IV-X11/xdisplay.h>
#include <IV-X11/xcanvas.h>


static const char* page_width_attrib = "pagewidth";
static const char* page_height_attrib = "pageheight";
static const char* grid_x_incr = "gridxincr";
static const char* grid_y_incr = "gridyincr";

DemKit* DemKit::_demkit = nil;

implementActionCallback(DemKit)

DemKit::DemKit() {
  _camwin = nil;
}

void DemKit::SetEditor(DemEditor* ed) {
    _ed = ed;
}

DemEditor* DemKit::GetEditor() {
    return (DemEditor*) _ed;
}

Interactor* DemKit::Interior() {
    return ((DemEditor*)_ed)->Interior();
}

void DemKit::Init(OverlayComp* comp, const char* name) {
    InitMembers(comp);
    InitViewer();
    InitLayout(name);
}

void DemKit::InitViewer () {
    Catalog* catalog = unidraw->GetCatalog();

    const char* page_w = catalog->GetAttribute(page_width_attrib);
    const char* page_h = catalog->GetAttribute(page_height_attrib);
    const char* x_incr = catalog->GetAttribute(grid_x_incr);
    const char* y_incr = catalog->GetAttribute(grid_y_incr);

    GraphicView* view = (GraphicView*)_ed->GetComponent()->Create(COMPONENT_VIEW);
    _ed->GetComponent()->Attach(view);
    view->Update();

    /*
     * These statements had to be moved down here to workaround
     * a strange cfront 3.0 bug.
     */
    const float w = round(atof(page_w) * ivinches);
    const float h = round(atof(page_h) * ivinches);

    OverlayPage* page = new OverlayPage(w, h);
    Grid* grid = new Grid(w, h, atof(x_incr), atof(y_incr));
    grid->Visibility(false);

    _ed->SetViewer(new DemViewer(_ed, view, page, grid));
}

void DemKit::InitLayout(const char* name) {
    if (_ed->GetWindow() == nil) {
	const LayoutKit& layout = *LayoutKit::instance();
	Glyph* menus = MakeMenus();
	Glyph* states = MakeStates();
	Glyph* toolbar = MakeToolbar();
	if (states)
	    menus->append(states);
	Target* viewer = 
	    new Target(Interior(), TargetPrimitiveHit);
	toolbar->append(layout.vcenter(viewer));
	menus->append(toolbar);
	ManagedWindow* w = new ApplicationWindow(_ed);
	_ed->SetWindow(w);
	Style* s = new Style(Session::instance()->style());
	s->alias(name);
	w->style(s);
	_ed->body(menus);
	_ed->GetKeyMap()->Execute(CODE_SELECT);
    }
}

Glyph* DemKit::MakeStates() {
#if 0
    WidgetKit& wk = *WidgetKit::instance();
    LayoutKit& lk = *LayoutKit::instance();
    Button* up_btn = wk.push_button("Up", new ActionCallback(DemEditor)(
	(DemEditor*)_ed, &DemEditor::CameraUp
    ));
    Button* dn_btn = wk.push_button("Down", new ActionCallback(DemEditor)(
	(DemEditor*)_ed, &DemEditor::CameraDown
    ));
    Button* yl_btn = wk.push_button("Yaw Left", new ActionCallback(DemEditor)(
	(DemEditor*)_ed, &DemEditor::CameraYawLeft
    ));
    Button* yr_btn = wk.push_button("Yaw Right", new ActionCallback(DemEditor)(
	(DemEditor*)_ed, &DemEditor::CameraYawRight
    ));
    Glyph* g = lk.hbox(lk.hglue(),
		       lk.vcenter(up_btn),
		       lk.vcenter(dn_btn),
		       lk.hglue(),
		       lk.vcenter(yl_btn),
		       lk.vcenter(yr_btn),
		       lk.hglue()
		   );
    return wk.outset_frame(lk.margin(g, 10));
#endif
    return nil;
}

void DemKit::SetXIds(vtkRenderWindow* renwin, Canvas* canvas)
{
    renwin->SetWindowId((void*)canvas->rep()->copybuffer_);
    renwin->SetDisplayId((void*)Session::instance()->default_display()->rep()->display_);
}

Glyph* DemKit::MakeMenus()
{
    LayoutKit& lk = *LayoutKit::instance();
    Menu* menubar_ = WidgetKit::instance()->menubar();
    
    menubar_->append_item(MakeFileMenu());
    menubar_->append_item(MakeRepMenu());
    menubar_->append_item(MakeShadeMenu());
    menubar_->append_item(MakeCameraMenu());
    menubar_->append_item(MakeCaptureMenu());
#ifdef HAVE_ACE
    menubar_->append_item(MakeVehicleMenu());
#endif

    Resource::ref(menubar_);
    return LayoutKit::instance()->vbox(lk.hnatural(menubar_, 300));
}

MenuItem * DemKit::MakeFileMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    Style * style = new Style(Session::instance()->style());
    
    MenuItem *mbi = kit.menubar_item(kit.label("File"));
    mbi->menu(kit.pulldown());

    MakeMenu(mbi, new DemLoadDemCmd(new ControlInfo("Open...", 
						    KLBL_VIEWCOMP, CODE_VIEWCOMP)),
	     "Open DEM...   ");
#if 0
    MakeMenu(mbi, new DemLoadLODDemCmd(new ControlInfo("Open...")),
	     "Open LOD DEM...   ");
#endif
    style->attribute("subcaption", "Import map features from file:");
    style->attribute("open", "Import");
    ImportChooser* ichooser = new ImportChooser(String("."), WidgetKit::instance(), style);
    Resource::ref(ichooser);
    MakeMenu(mbi, new DemImportFCCmd(new ControlInfo("Import Map Features...",
						  KLBL_IMPORT,
						  CODE_IMPORT),
				   ichooser),
	     "Import Map Features...   ");
    MakeMenu(mbi, new DemLoadFCCmd(new ControlInfo("Load Map Features...")),
	     "Load Map Features...     ");
    MakeMenu(mbi, new DemExportVRMLCmd(new ControlInfo("Export", "^X", "\030")),
	     "Export VRML...");
    MakeMenu(mbi, new MapAboutCmd(new ControlInfo("About ivmaps demviewer", KLBL_ABOUT, CODE_ABOUT)),
	     "About ivmaps demviewer");
    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new OvQuitCmd(new ControlInfo("Quit", KLBL_QUIT, CODE_QUIT)),
	     "Quit          ");
    return mbi;
}

MenuItem * DemKit::MakeRepMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("Rep"));
    mbi->menu(kit.pulldown());

    MakeMenu(mbi, new DemRepresentationCmd(new ControlInfo("Points"),
					   DemRepresentationCmd::POINTS),
	     "Points               ");
    MakeMenu(mbi, new DemRepresentationCmd(new ControlInfo("Wireframe"),
					   DemRepresentationCmd::WIREFRAME),
	     "Wireframe            ");
    MakeMenu(mbi, new DemRepresentationCmd(new ControlInfo("Surface"),
					   DemRepresentationCmd::SURFACE),
	     "Surface              ");
    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new DemBackfaceCullCmd(new ControlInfo("Backface Culling On"),
					 true),
	     "Backface Culling On  ");
    MakeMenu(mbi, new DemBackfaceCullCmd(new ControlInfo("Backface Culling Off"),
					 false),
	     "Backface Culling Off ");

    return mbi;
}

MenuItem * DemKit::MakeShadeMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("Render"));
    mbi->menu(kit.pulldown());

    MakeMenu(mbi, new DemShadingCmd(new ControlInfo("Flat"),
					   DemShadingCmd::FLAT),
	     "Flat Shading        ");
    MakeMenu(mbi, new DemShadingCmd(new ControlInfo("Gouraud"),
					   DemShadingCmd::GOURAUD),
	     "Gouraud Shading     ");
    MakeMenu(mbi, new DemShadingCmd(new ControlInfo("Phong"),
					   DemShadingCmd::PHONG),
	     "Phong Shading       ");
    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new DemNoCullCmd(new ControlInfo("No Culling")),
	     "No Culling          ");
    MakeMenu(mbi, new DemNoCullCmd(new ControlInfo("2D Footprint Culling")),
	     "2D Footprint Culling");

    return mbi;
}

MenuItem * DemKit::MakeCameraMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("Camera"));
    mbi->menu(kit.pulldown());

    MakeMenu(mbi, new DemShowCamStateCmd(new ControlInfo("Show State")),
	     "Show State");
    MakeMenu(mbi, new DemHideCamStateCmd(new ControlInfo("Hide State")),
	     "Hide State");
    MakeMenu(mbi, new DemCameraUpCmd(new ControlInfo("Up", "u", "u")),
	     "Up        ");
    MakeMenu(mbi, new DemCameraDownCmd(new ControlInfo("Down", "d", "d")),
	     "Down      ");
    MakeMenu(mbi, new DemCameraDollyCmd(new ControlInfo("Dolly In"), 1.1),
	     "Dolly In  ");
    MakeMenu(mbi, new DemCameraDollyCmd(new ControlInfo("Dolly Out"), 0.9),
	     "Dolly Out ");
    MakeMenu(mbi, new DemCameraYawLeftCmd(new ControlInfo("Yaw Left")),
	     "Yaw Left  ");
    MakeMenu(mbi, new DemCameraYawRightCmd(new ControlInfo("Yaw Right")),
	     "Yaw Right ");
    //MakeMenu(mbi, new DemCameraResetCmd(new ControlInfo("Reset")),
    //     "Reset     ");

    return mbi;
}

MenuItem * DemKit::MakeCaptureMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("Capture"));
    mbi->menu(kit.pulldown());

    MakeMenu(mbi, new DemStartCaptureCmd(new ControlInfo("Start Frame Capture")),
	     "Start Frame Capture ");
    MakeMenu(mbi, new DemStopCaptureCmd(new ControlInfo("Stop Frame Capture")),
	     "Stop Frame Capture ");
    MakeMenu(mbi, new DemToggleRteCaptureCmd(new ControlInfo("Toggle Route Capture", "R", "R")),
	     "Toggle Route Capture ");

    return mbi;
}

MenuItem * DemKit::MakeVehicleMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("Vehicle"));
    mbi->menu(kit.pulldown());

    MakeMenu(mbi, new DemVehicleInitCmd(new ControlInfo("Initialize")),
	     "Initialize ");
    MakeMenu(mbi, new DemVehicleGasCmd(new ControlInfo("Gas", "G", "G")),
	     "Gas        ");
    MakeMenu(mbi, new DemVehicleBrakeCmd(new ControlInfo("Brake", "B", "B")),
	     "Brake      ");
    MakeMenu(mbi, new DemVehicleForwardCmd(new ControlInfo("Forward", "f", "f")),
	     "Forward    ");
    MakeMenu(mbi, new DemVehicleBackwardCmd(new ControlInfo("Backward", "b", "b")),
	     "Backward   ");
    MakeMenu(mbi, new DemVehicleTurnLeftCmd(new ControlInfo("Turn Left", "l", "l")),
	     "Turn Left  ");
    MakeMenu(mbi, new DemVehicleTurnRightCmd(new ControlInfo("Turn Right", "r", "r")),
	     "Turn Right ");
    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new DemNewViewCmd(new ControlInfo("New View")),
	     "New View   ");

    return mbi;
}

Glyph* DemKit::MakeToolbar() {
    LayoutKit& lk = *LayoutKit::instance();
    return lk.hbox(new Background(
		    lk.vcenter(
			lk.vglue(300)
		    ),
		    unidraw->GetCatalog()->FindColor("#aaaaaa")
    ));
}

DemKit* DemKit::Instance() {
    if (!_demkit)
	_demkit = new DemKit();
    return _demkit;
}

void DemKit::ShowCameraStateWindow() {
  if (!_camwin) {
    DemEditor* ed = GetEditor();
    BoundedValueObserver* camxobs = new BoundedValueObserver(ed->CameraX(),
							     "X: ");
    BoundedValueObserver* camyobs = new BoundedValueObserver(ed->CameraY(),
							     "Y: ");
    BoundedValueObserver* camzobs = new BoundedValueObserver(ed->CameraZ(),
							     "Z: ");
    BoundedValueObserver* fptxobs = new BoundedValueObserver(ed->FocalPtX(),
							     "X: ");
    BoundedValueObserver* fptyobs = new BoundedValueObserver(ed->FocalPtY(),
							     "Y: ");
    BoundedValueObserver* fptzobs = new BoundedValueObserver(ed->FocalPtZ(),
							     "Z: ");
    BoundedValueObserver* clipfobs = new BoundedValueObserver(ed->ClipFront(),
							     "Front: ");
    BoundedValueObserver* clipbobs = new BoundedValueObserver(ed->ClipBack(),
							     "Back: ");
    BoundedValueObserver* vwupxobs = new BoundedValueObserver(ed->ViewUpX(),
							     "X: ");
    BoundedValueObserver* vwupyobs = new BoundedValueObserver(ed->ViewUpY(),
							     "Y: ");
    BoundedValueObserver* vwupzobs = new BoundedValueObserver(ed->ViewUpZ(),
							     "Z: ");
    BoundedValueObserver* angleobs = new BoundedValueObserver(ed->ViewAngle(),
							     "View Angle: ");
    BoundedValueObserver* distobs = new BoundedValueObserver(ed->Distance(),
							     "Distance: ");
    WidgetKit& wk = *WidgetKit::instance();
    LayoutKit& lk = *LayoutKit::instance();

    PolyGlyph * mainglyph = lk.vbox();
    PolyGlyph* vb = lk.vbox();
    vb->append(lk.hbox(wk.label("Position: "),
		       camxobs,
		       lk.hglue(10),
		       camyobs,
		       lk.hglue(10),
		       camzobs));
    vb->append(lk.vspace(10));
    vb->append(
	       lk.hbox(wk.label("Focal Point: "),
		       fptxobs,
		       lk.hglue(10),
		       fptyobs,
		       lk.hglue(10),
		       fptzobs));
    vb->append(lk.vspace(10));
    vb->append(distobs);
    vb->append(lk.vspace(10));
    vb->append(
	       lk.hbox(wk.label("Clipping Range: "),
		       clipfobs,
		       lk.hglue(10),
		       clipbobs
		       ));
    vb->append(lk.vspace(10));
    vb->append(
	       lk.hbox(wk.label("View Up Vector: "),
		       vwupxobs,
		       lk.hglue(10),
		       vwupyobs,
		       lk.hglue(10),
		       vwupzobs));
    vb->append(lk.vspace(10));
    vb->append(angleobs);
    Action* disaction = new ActionCallback(DemKit)
      (this, &DemKit::HideCameraStateWindow);
    Button* dis = wk.push_button(wk.label("Dismiss"), disaction);
    vb->append(lk.vspace(10));

    mainglyph->append(
	 wk.outset_frame(
	    lk.vbox(
		    lk.vspace(10),
		    lk.hcenter(wk.fancy_label("3D Camera State")),
		lk.natural(
		    lk.tmargin(
			lk.vbox(lk.hcenter(vb),lk.hcenter(dis))
			,
			10
		    ),
		    400, 180
		)
	    )
	)
    );

    _camwin = new ApplicationWindow(mainglyph);
  }
  if (!_camwin->is_mapped())
    _camwin->map();
}

void DemKit::HideCameraStateWindow() {
  if (_camwin && _camwin->is_mapped())
    _camwin->unmap();
}

#endif
