/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <OverlayUnidraw/ovclasses.h>

#include <MapUnidraw/mapabout.h>
#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapcreator.h>
#include <MapUnidraw/mapexport.h>
#include <MapUnidraw/mappoint.h>
#include <MapUnidraw/mapproj.h>
#include <MapUnidraw/mapps.h>
#include <MapUnidraw/mapscripts.h>
#include <MapUnidraw/mapstatecmds.h>
#include <MapUnidraw/mapviews.h>
#include <MapUnidraw/maproute.h>
#include <FrameUnidraw/framecreator.h>
#include <Unidraw/catalog.h>


/*****************************************************************************/

MapCreator::MapCreator () { }

void* MapCreator::Create (
    ClassId id, istream& in, ObjectMap* objmap, int objid
) {
    switch (id) {
        case MAPABOUT_CMD:         CREATE(MapAboutCmd, in,objmap,objid);
        case MAPPROJECTION_CMD:    CREATE(MapProjectionCmd, in, objmap, objid);
        case MAPLIB_CMD:           CREATE(MapLibCmd, in, objmap, objid);
        case MAPCOV_CMD:           CREATE(MapCovCmd, in, objmap, objid);
        case MAPFCLASS_CMD:        CREATE(MapFClassCmd, in, objmap, objid);
        case MAPSELECTION_CMD:     CREATE(MapSelectionCmd, in, objmap, objid);
        case POINT_CMD:            CREATE(PointCmd, in, objmap, objid);
        case CONTOUR_CMD:          CREATE(ContourCmd, in, objmap, objid);
        case MAPEXPORT_CMD:        CREATE(MapExportCmd, in, objmap, objid);
        case MAPOPEN_CMD:          CREATE(MapOpenCmd, in, objmap, objid);
        case MAPSHOWALL_CMD:       CREATE(MapShowAllCmd, in, objmap, objid);
        case MAPUPDATEMENUS_CMD:   CREATE(MapUpdateMenusCmd, in, objmap, objid);

        default:                   return FrameCreator::Create(id, in,objmap,objid);
    }
}

void* MapCreator::Create (ClassId id) {
    if (id == MAPFEATURE_VIEW)   return new MapFeatureView;
    if (id == MAPFCLASS_VIEW)    return new MapFClassView;
    if (id == MAPCOV_VIEW)       return new MapCovView;
    if (id == MAPLIB_VIEW)       return new MapLibView;
    if (id == MAPDB_VIEW)        return new MapDbView;
    if (id == MAP_IDRAW_VIEW)    return new MapIdrawView;
    if (id == MAP_ROUTE_VIEW)    return new MapRouteView;

    if (id == MAPFEATURE_SCRIPT) return new MapFeatureScript;
    if (id == MAPFCLASS_SCRIPT)  return new MapFClassScript;
    if (id == MAPCOV_SCRIPT)     return new MapCovScript;
    if (id == MAPLIB_SCRIPT)     return new MapLibScript;
    if (id == MAPDB_SCRIPT)      return new MapDbScript;
    if (id == MAP_IDRAW_SCRIPT)  return new MapIdrawScript;
    if (id == MAP_ROUTE_SCRIPT)  return new MapRouteScript;

    if (id == MAPFEATURE_PS)     return new MapFeaturePS;
    if (id == MAPFCLASS_PS)      return new OverlaysPS;
    if (id == MAPCOV_PS)         return new OverlaysPS;
    if (id == MAPLIB_PS)         return new OverlaysPS;
    if (id == MAPDB_PS)          return new OverlaysPS;

    if (id == MAPFEATURE_RAW)    return new MapFeatureRawScript;
    if (id == MAPFCLASS_RAW)     return new MapFClassRawScript;

    return FrameCreator::Create(id);
}
