/*
 * Copyright 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef routecomps_h
#define routecomps_h

#include <MapUnidraw/mapviews.h>
#include <MapUnidraw/mapinspect.h>
#include <MapUnidraw/mapscripts.h>
#include <Unidraw/Tools/tool.h>

class MapElement;

class MapRouteComp : public MapFeatureComp {
public:
  MapRouteComp(MapElement* =nil);
  MapRouteComp(istream& in, OverlayComp* parent =nil);
  virtual ~MapRouteComp();

  virtual Component* Copy();
  virtual ClassId GetClassId();
  virtual boolean IsA(ClassId);

protected:
  ParamList* GetParamList();
  void GrowParamList(ParamList*);
  static ParamList* _maproute_params;

};

class MapRouteView : public MapFeatureView {
public:
  MapRouteView(MapRouteComp* =nil);
  virtual ~MapRouteView();

  virtual ClassId GetClassId();
  virtual boolean IsA(ClassId);

  virtual Graphic* GetGraphic();
  virtual void Update();
  virtual Graphic* HighlightGraphic();
  virtual void GetVertices(Coord*&, Coord*&, int&);
protected:
  virtual void CreateHandles();
};

class MapRouteScript : public MapFeatureScript {
public:
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
    MapRouteScript(MapRouteComp* = nil);

    virtual boolean Definition(ostream&);
    static int ReadPoints(istream&, void*, void*, void*, void*);
};

class RouteTool : public Tool {
public:
  RouteTool(ControlInfo* ci);

  virtual Manipulator* CreateManipulator(Viewer*,Event&,Transformer*);
  virtual Command* InterpretManipulator(Manipulator*);
};

class PickRouteTool : public MapInspectTool {
public:
  PickRouteTool(ControlInfo* ci);

protected:
  virtual void DoInspect(GraphicView*, Viewer*);
};

#endif
