/*
 * Copyright 1994, 1995 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <VpfUnidraw/vpfeditor.h>
#include <VpfUnidraw/vpfgaz.h>
#include <VpfUnidraw/vpfkit.h>
#include <MapUnidraw/mapstates.h>
#include <Vpf/vpfdb.h>
#include <Vpf/vpflib.h>

#include <IV-look/choice.h>
#include <IV-look/fbrowser.h>
#include <IV-look/field.h>
#include <IV-look/dialogs.h>
#include <IV-look/kit.h>
#include <InterViews/action.h>
#include <InterViews/cursor.h>
#include <InterViews/font.h>
#include <InterViews/hit.h>
#include <InterViews/layout.h>
#include <InterViews/style.h>
#include <InterViews/target.h>
#include <InterViews/window.h>
#include <OS/string.h>
#include <stdio.h>

declareActionCallback(VpfGazetteer)
implementActionCallback(VpfGazetteer)
declareFieldEditorCallback(VpfGazetteer)
implementFieldEditorCallback(VpfGazetteer)

VpfGazetteer::VpfGazetteer(VpfKit* vk, Style* s)
: Dialog(nil, s)
{
    _vpfkit = vk;
    build();
}

void VpfGazetteer::build() {
    DialogKit& dialog = *DialogKit::instance();
    LayoutKit& layout = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    Style* s = kit.style();

    _field = dialog.field_editor("", s, new FieldEditorCallback(VpfGazetteer)
				 (this, &VpfGazetteer::fquery,
				  &VpfGazetteer::fquit));
    _browser = new FileBrowser(WidgetKit::instance(),
			       new ActionCallback(VpfGazetteer)(
				   this, &VpfGazetteer::query
			       ), nil);
    _queryb = kit.default_button("Query", 
				 new ActionCallback(VpfGazetteer)(
				     this, &VpfGazetteer::query
				 ));
    Button* quitb = kit.push_button("Dismiss",
				    new ActionCallback(VpfGazetteer)(
					this, &VpfGazetteer::quit
				    ));
    Button* idxb = kit.push_button("Index",
				    new ActionCallback(VpfGazetteer)(
					this, &VpfGazetteer::tileindex
				    ));

    const Font* f = kit.font();
    FontBoundingBox bbox;
    f->font_bbox(bbox);
    Coord height = 6 * (bbox.ascent() + bbox.descent()) + 1.0;
    Coord width = 80 * f->width('M') + 3.0;

    Glyph* g = layout.vbox();
    g->append(layout.hcenter(kit.fancy_label("Gazetteer")));
    g->append(layout.vglue(10,0,3));
    g->append(layout.hcenter(kit.label("Enter place name:")));
    g->append(layout.vglue(5,0,2));
    g->append(layout.hcenter(layout.hfixed(_field, 200)));
    g->append(layout.vglue(5,0,2));
    g->append(
	layout.hcenter(
	layout.hbox(
	    kit.inset_frame(
		layout.margin(
		    layout.natural_span(_browser, width, height), 1.0
		)
	    ),
	    layout.hspace(4),
	    kit.vscroll_bar(_browser->adjustable())
	))
    );
    g->append(layout.vspace(15.0));
    g->append(
	layout.hcenter(
	layout.hbox(
	    layout.hglue(10.0),
	    layout.vcenter(_queryb),
	    layout.hglue(10.0, 0.0, 5.0),
	    layout.vcenter(quitb),
	    layout.hglue(10.0),
	    layout.vcenter(idxb),
	    layout.hglue(10.0)
	))
    );
    body(
	layout.back(
	    layout.vcenter(kit.outset_frame(layout.margin(g, 5.0)), 1.0),
	    new Target(nil, TargetPrimitiveHit)
	)
    );
    append_input_handler(_field);
    focus(_field);
}

void VpfGazetteer::query() {
    if (_field->text()->length() > 0) {
	VpfDatabase* db = ((VpfEditor*)_vpfkit->GetEditor())->GetMapDatabase();
	MapLibState* state = ((VpfEditor*)_vpfkit->GetEditor())->libstate();
	VpfLibrary* lib = db->library(state->maplib());
	if (lib && lib->gazette()) {
	    char** matches;
	    char** types;
	    float* x;
	    float* y;
	    int hits = lib->gazette_search(_field->text()->string(),
				       matches, types, x, y);
	    if (hits > 0) {
		LayoutKit& layout = *LayoutKit::instance();
		WidgetKit& kit = *WidgetKit::instance();
		int count = _browser->count();
		for (int g = count-1; g >= 0 ; g--) {
		    _browser->remove(g);
		    _browser->remove_selectable(g);
		}
		for (int i = 0; i < hits; i++) {
		    char line[256];
		    sprintf(line, "%30s %25s   %0.6f,%0.6f", matches[i], types[i],
			    y[i], x[i]);
		    delete [] matches[i];
		    delete [] types[i];
		    Glyph* name = kit.label(line);
		    Glyph* label = new Target(
			layout.h_margin(name, 3.0, 0.0, 0.0, 15.0, fil, 0.0),
			TargetPrimitiveHit);
		    TelltaleState* t = new TelltaleState(TelltaleState::is_enabled);
		    _browser->append_selectable(t);
		    _browser->append(new ChoiceItem(t, label, kit.bright_inset_frame(label)));
		    _browser->refresh();
		}
		delete [] matches;
		delete [] types;
		delete [] x;
		delete [] y;
	    }
	}
    }
}

void VpfGazetteer::quit() {
    dismiss(true);
}

void VpfGazetteer::fquery(FieldEditor*) {
    query();
}

void VpfGazetteer::fquit(FieldEditor*) {
    dismiss(true);
}

void VpfGazetteer::show() {
    printf("show\n");
}

void VpfGazetteer::tileindex() {
  VpfDatabase* db = ((VpfEditor*)_vpfkit->GetEditor())->GetMapDatabase();
  MapLibState* state = ((VpfEditor*)_vpfkit->GetEditor())->libstate();
  VpfLibrary* lib = db->library(state->maplib());
  if (lib && lib->gazette() && lib->tiled()) {
    ((VpfEditor*)_vpfkit->GetEditor())->GetWindow()->cursor(hourglass);
    lib->gaz_tileindex();
    ((VpfEditor*)_vpfkit->GetEditor())->GetWindow()->cursor(arrow);
  }
}
